-- SETTINGS
-- Tails
local TAILS_FLY_COUNT = 3

-- Control types
local CTRL_SIMPLE = 3
local CTRL_DIRECTIONCHAR = 2
local CTRL_SIMPLESTRAFE = 1
local CTRL_STRAFE = 0

function A_CheckExistence(p)
	if p
	and p.valid
	and p.mo
	and p.mo.valid
		return true
	else
		return false
	end
end

function A_PlayerInit(p)
	p.init = true
	p.prevbuttons = p.cmd.buttons
	p.spinbutton = p.cmd.buttons & BT_USE
	p.shieldwindow = 0
	p.usedshield = false
	if p.kr_freeaim == nil
		p.kr_freeaim = 1
	end
	if p.kr_uncurltoggle == nil
		p.kr_uncurltoggle = true
	end
	if p.kr_superpowers == nil
		p.kr_superpowers = true
	end
	A_ResetAbilities(p)
	return
end

-- Checks if two things overlap vertically
function A_CheckCollisionHeight(a, b)
	if not (a.valid and b.valid)
		return false
	end
	
	if a.z + a.momz + a.height < b.z + b.momz
	or b.z + b.momz + b.height < a.z + a.momz 
		return false
	end
	return true
end

function A_CheckNoControl(p)
	if (P_PlayerInPain(p)
	or p.state == S_PLAY_PAIN
	or (p.playerstate == PST_DEAD)
	or p.powers[pw_carry])
		return true
	else
		return false
	end
end

-- Checks if you're one of the game's default characters, so we don't go too crazy with you
function A_CheckStockChar(p)
	if p.mo.skin == "sonic"
	or p.mo.skin == "tails"
	or p.mo.skin == "knuckles"
	or p.mo.skin == "amy"
	or p.mo.skin == "fang"
	or p.mo.skin == "metalsonic"
		return true
	else
		return false
	end
end

function A_GetCtrlStyle(p)
    -- Returns 3 if "Simple" control scheme is enabled.
    -- Returns 2 if directionchar is set to movement
    -- Returns 1 if directionchar is set to camera, but Simple control scheme is enabled, AKA Analog Strafe
    -- Returns 0 if directionchar is set to camera, AKA Strafe
    return ((p.pflags & PF_ANALOGMODE) and 1 or 0) | ((p.pflags & PF_DIRECTIONCHAR) and 2 or 0)
end

function A_GetMoveDirection(p)
	local direction = p.mo.angle
	
	if A_GetCtrlStyle(p) != CTRL_SIMPLE
		direction = $ + R_PointToAngle2(0, 0, p.cmd.forwardmove * FRACUNIT, p.cmd.sidemove * -FRACUNIT)
	end
	
	return direction
end

function A_ResetFlightCount(p)
	p.mo.flycount = TAILS_FLY_COUNT
end

function A_ResetAbilities(p)
	if not (p and p.mo and p.mo.valid)
		return
	end
	
	local mo = p.mo
	
	-- Reset whether you've curled up mid-air with spin
	mo.aircurled = false
	
	-- Reset whether you've used your shield ability
	p.usedshield = false
	
	-- The rest of this stuff is for modified stock characters only
	if not A_CheckStockChar(p)
		return
	end
	
	-- Reset whether Sonic has double-jumped (using PF_THOKKED stops roll on landing)
	mo.doublejumped = false
	-- Reset how many frames Sonic has charged drop dash
	mo.dropcharge = $ or 0
	-- Reset how long Tails has been flying
	mo.flytimer = 0
	-- Reset how many more times Tails can boost
	A_ResetFlightCount(p)
	-- Reset the 'used ability' flag
	p.pflags = $ & ~PF_THOKKED
end

function A_CheckCanAircurl(p)
	-- Find a reason not to be able to aircurl
	if P_IsObjectOnGround(p.mo) -- Not in the air
	or (p.charability2 != CA2_SPINDASH) -- Can't spin
	or (p.pflags & PF_SPINNING) -- Already spinning
	or (p.mo.state == S_PLAY_ROLL) -- Already curled up
	or (p.mo.state == S_PLAY_JUMP) -- Already curled up pt. 2
	or (p.prevbuttons & BT_USE) -- Already holding spin
	or (p.mo.aircurled) -- Already aircurled
		return false
	else
		return true
	end
end

function A_Aircurl(p)
	p.mo.state = S_PLAY_JUMP
	p.mo.aircurled = true
	p.pflags = $ | PF_JUMPED & ~PF_GLIDING & ~PF_THOKKED
	S_StartSound(p.mo, sfx_curl)
	return true
end

function A_Uncurl(p)
	local mo = p.mo
	
	-- Stop spinning
	p.pflags = $ & ~PF_SPINNING
	
	-- Walk or run
	if FixedHypot(mo.momx, mo.momy) >= p.runspeed
		mo.state = S_PLAY_RUN
	else
		mo.state = S_PLAY_WALK
	end
end