-- SETTINGS
-- Initial flight lift
local FLY_ASCENDSPEED = (53/20)*FRACUNIT
-- Maximum flight upward speed (unused)
local FLY_MAXMOMZ = 10 * FRACUNIT
-- Number of tics you spend ascending
local FLY_ASCENDTICS = 7
-- Number of tics you spend ascending when swimming
local FLY_ASCENDTICS_SWIM = 4
-- Minimum time between flight boosts
local FLY_SPAMTICS = 10
-- How fast you fall at first
local FLY_HOVERSPEED = -(7/2) * FRACUNIT
-- How fast you fall when you're exhausted
local FLY_HOVERSPEED_TIRED = -(11/2) * FRACUNIT
-- How many tics it takes you to enter the exhausted state
local FLY_TIREDTIME = 55
-- How many tics it takes to drop out of the air entirely
local FLY_MAXTIME = 85

-- Tails flight activated
function A_TailsFly(player)
	if not A_CheckExistence(player)
	or not (player.mo.skin == "tails")
	or not (player.pflags & PF_JUMPED)
		return false
	end
	
	local mo = player.mo
	local speed = FixedHypot(mo.momx, mo.momy)
	local momangle = R_PointToAngle2(0, 0, mo.momx, mo.momy)
	
	if mo.flycount <= 0
	or (mo.flytimer > 0 and mo.flytimer < FLY_SPAMTICS)
		return true
	end
	
	-- Momentum
	mo.momz = $ / 2
	P_Thrust(mo, momangle + ANGLE_180, speed / 4)
	
	-- Flags, states
	mo.aircurled = false
	mo.flytimer = 0
	mo.flycount = $ - 1
	mo.flylock = FLY_SPAMTICS
	player.pflags = $ & ~PF_JUMPED
	player.pflags = $ & ~PF_SPINNING
	mo.state = S_PLAY_FLY
	
	-- Sound effect
	if not (mo.eflags & MFE_UNDERWATER)
		S_StartSound(mo, sfx_putput)
	end
	return true
end

function A_TailsHover(player)
	local mo = player.mo

	-- Declare flight-related variables
	local hoverspeed = FLY_HOVERSPEED
	local flip = P_MobjFlip(mo)
	
	-- Change state to tired to show when you can boost again
	if mo.flytimer >= FLY_SPAMTICS
		if mo.state == S_PLAY_FLY
			mo.state = S_PLAY_FLY_TIRED
		end
	end
	
	-- Change sound when you hit tired physics
	if mo.flytimer < FLY_TIREDTIME
		if mo.flytimer % 10 == 1
		and (mo.state == S_PLAY_FLY or mo.state == S_PLAY_FLY_TIRED)
			S_StartSound(mo, sfx_s3kba)
		end
	else
		if mo.flytimer % 5 == 1
		and mo.state == S_PLAY_FLY_TIRED
			S_StartSound(mo, sfx_s3kbb)
		end
		
		-- Hover at tired hovering speed
		hoverspeed = FLY_HOVERSPEED_TIRED
	end
	
	-- Decide on vertboost duration
	local ascendtics = FLY_ASCENDTICS
	if mo.eflags & MFE_UNDERWATER
		ascendtics = FLY_ASCENDTICS_SWIM
	end
	local speed = FixedHypot(mo.momx, mo.momy)
	
	//print(speed/FRACUNIT)
	
	-- Fall no faster than your hover speed
	if (flip == 1 and mo.momz < hoverspeed) or (flip == -1 and mo.momz > -hoverspeed)
		P_SetObjectMomZ(mo, hoverspeed, false)
	end
	//P_SetObjectMomZ(mo, min(flippedmomz, hoverspeed))
	
	-- If you're still in vertboost, apply boost
	if (mo.flytimer <= ascendtics)
		-- Calculate boost to apply
		local ascendspeed = FLY_ASCENDSPEED
		if mo.eflags&MFE_UNDERWATER
			ascendspeed = $ * 2/3
		end
		
		P_SetObjectMomZ(mo, ascendspeed, true)
	end
	
	if mo.flytimer >= FLY_MAXTIME
	and (mo.state == S_PLAY_FLY or mo.state == S_PLAY_FLY_TIRED)
		mo.flytimer = 0
		mo.state = S_PLAY_FALL
	else
		mo.flytimer = $ + 1
	end
end
